#ifndef _LLOG_H_
#define _LLOG_H_

#include <stddef.h>
#include <stdint.h>
#include <stdarg.h>

#include "sysenv.h"
#if _OS_WIN

#elif _OS_LINUX

#else
#error "Not Supported Operator System"
#endif

typedef uint16_t log_mask_t;

#define LOG_MASK_ENDLINE	(1 << 0)
#define LOG_MASK_COLORS		(1 << 1)
#define LOG_MASK_LEVEL		(1 << 2)
#define LOG_MASK_FILE		(1 << 3)
#define LOG_MASK_FUNC		(1 << 4)
#define LOG_MASK_LINE		(1 << 5)
#define LOG_MASK_MODE		(1 << 6)

#define LOG_MASK_SECOND		(1 << 8)
#define LOG_MASK_MINUTE		(1 << 9)
#define LOG_MASK_HOUR		(1 << 10)
#define LOG_MASK_DAY		(1 << 11)
#define LOG_MASK_MONTH		(1 << 12)
#define LOG_MASK_YEAR		(1 << 13)

#define LOG_FILE_SIMPLE		(1 << 15)

#define LOG_MASK_TIME_HMS		(LOG_MASK_SECOND | LOG_MASK_MINUTE | LOG_MASK_HOUR)
#define LOG_MASK_TIME_YMD		(LOG_MASK_DAY | LOG_MASK_MONTH | LOG_MASK_YEAR)
#define LOG_MASK_TIME_ALL		(LOG_MASK_TIME_HMS | LOG_MASK_TIME_YMD)

#define LOG_MASK_ALL_BASE		(LOG_MASK_LEVEL | LOG_MASK_LINE | LOG_MASK_FUNC)
#define LOG_MASK_ALL_MODE		(LOG_MASK_MODE | LOG_MASK_ALL_BASE)
#define LOG_MASK_ALL_FILE		(LOG_MASK_FILE | LOG_MASK_ALL_BASE)

#define LOG_MASK_ALL_DETAIL		(LOG_MASK_MODE | LOG_MASK_FILE | LOG_MASK_ALL_BASE)
#define LOG_MASK_BASE_STDOUT	(LOG_MASK_COLORS | LOG_MASK_ALL_DETAIL | LOG_MASK_TIME_HMS)
#define LOG_MASK_BASE_FILE		(LOG_MASK_ALL_DETAIL | LOG_MASK_TIME_ALL)
#define LOG_MASK_BASE_DEFAULT	(LOG_MASK_BASE_STDOUT | LOG_FILE_SIMPLE)

#ifdef __cplusplus
extern "C" {
#endif

enum { _LOG_TRACE, _LOG_DEBUG, _LOG_INFO, _LOG_WARN, _LOG_ERROR, _LOG_FATAL };

#define log_head_trace( log, szlog) log_fill_head_str(_LOG_TRACE, log, szlog, LOG_MASK_BASE_DEFAULT, __FILE__, __func__, __func__, __LINE__)
#define log_head_debug( log, szlog) log_fill_head_str(_LOG_DEBUG, log, szlog, LOG_MASK_BASE_DEFAULT, __FILE__, __func__, __func__, __LINE__)
#define log_head_info(  log, szlog) log_fill_head_str(_LOG_INFO,  log, szlog, LOG_MASK_BASE_DEFAULT, __FILE__, __func__, __func__, __LINE__)
#define log_head_warn(  log, szlog) log_fill_head_str(_LOG_WARN,  log, szlog, LOG_MASK_BASE_DEFAULT, __FILE__, __func__, __func__, __LINE__)
#define log_head_error( log, szlog) log_fill_head_str(_LOG_ERROR, log, szlog, LOG_MASK_BASE_DEFAULT, __FILE__, __func__, __func__, __LINE__)
#define log_head_fatal( log, szlog) log_fill_head_str(_LOG_FATAL, log, szlog, LOG_MASK_BASE_DEFAULT, __FILE__, __func__, __func__, __LINE__)
#define log_head(level, log, szlog) log_fill_head_str(level     , log, szlog, LOG_MASK_BASE_DEFAULT, __FILE__, __func__, __func__, __LINE__)

#define log_head_trace_ex( log, szlog, mode, mask) log_fill_head_str(_LOG_TRACE, log, szlog, mask, __FILE__, mode, __func__, __LINE__)
#define log_head_debug_ex( log, szlog, mode, mask) log_fill_head_str(_LOG_DEBUG, log, szlog, mask, __FILE__, mode, __func__, __LINE__)
#define log_head_info_ex(  log, szlog, mode, mask) log_fill_head_str(_LOG_INFO,  log, szlog, mask, __FILE__, mode, __func__, __LINE__)
#define log_head_warn_ex(  log, szlog, mode, mask) log_fill_head_str(_LOG_WARN,  log, szlog, mask, __FILE__, mode, __func__, __LINE__)
#define log_head_error_ex( log, szlog, mode, mask) log_fill_head_str(_LOG_ERROR, log, szlog, mask, __FILE__, mode, __func__, __LINE__)
#define log_head_fatal_ex( log, szlog, mode, mask) log_fill_head_str(_LOG_FATAL, log, szlog, mask, __FILE__, mode, __func__, __LINE__)
#define log_head_ex(level, log, szlog, mode, mask) log_fill_head_str(level     , log, szlog, mask, __FILE__, mode, __func__, __LINE__)


size_t log_fill_head_str(int level, char* log, size_t szlog,
	log_mask_t mask, const char* file, const char* mode, const char* func, int line);

#ifdef __cplusplus
}
#endif

/*
 * Copyright (c) 2020 rxi
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#endif /* _LLOG_H_ */